local MODNAME = 'MelodicVitals'
local ShivvAudio = LibStub('AceAddon-3.0'):GetAddon('ShivvAudio')
local MelodicVitals = ShivvAudio:NewModule(MODNAME, 'AceEvent-3.0')

local hpDB, pwrDB
local activeHP, activePWR = {}, {}

function MelodicVitals:OnInitialize()
	hpDB, pwrDB = ShivvAudio.db.profile.melodicVitals.healthDB, ShivvAudio.db.profile.melodicVitals.powerDB

	self:SetEnabledState(ShivvAudio.db.profile.enableMelodicVitals)
end

local function hasEntries(tbl)
	for entry in pairs(tbl) do
		return true -- only happens if theres an entry
	end
	return false
end

function MelodicVitals:OnEnable()
	-- clean out tracking data
	for unit in pairs(activeHP) do activeHP[unit] = nil end
	for unit in pairs(activePWR) do activePWR[unit] = nil end

	-- setup repeat timer local refs
	self:VitalRepeatSetupHP()
	self:VitalRepeatSetupPWR()

	for unit in pairs(hpDB) do
		if (hpDB[unit].enable) then
			activeHP[unit] = 0

			if (UnitExists(unit)) then
				self:UNIT_HEALTH(nil, unit) -- force update if unit currently exists
			end
		end
	end

	for unit in pairs(pwrDB) do
		if (pwrDB[unit].enable) then
			activePWR[unit] = 0
			if (UnitExists(unit)) then
				self:UNIT_POWER(nil, unit)
			end
		end
	end

	if (hasEntries(activeHP)) then
		self:RegisterEvent('UNIT_HEALTH')
	end

	if (hasEntries(activePWR)) then
		self:RegisterEvent('UNIT_POWER')
	end
end

function MelodicVitals:OnDisable()
	self:UnregisterEvent('UNIT_HEALTH')
	self:UnregisterEvent('UNIT_POWER')
end

do
	local GetTime, UnitHealth, UnitHealthMax = _G.GetTime, _G.UnitHealth, _G.UnitHealthMax
	local r, p, t

	function MelodicVitals:VitalRepeatSetupHP()
		r = ShivvAudio.db.profile.melodicVitals.repeatTimer
	end

	function MelodicVitals:UNIT_HEALTH(e, unit)
		if (not activeHP[unit]) then return end -- abort if not tracking

		p = UnitHealth(unit) / UnitHealthMax(unit)

		if (p < hpDB[unit].percent) then -- alert, its below threshold
			t = GetTime()

			if ((activeHP[unit] + r) < t and not UnitIsDeadOrGhost(unit)) then -- has it been long enough and are they living
				ShivvAudio:SoundHandler(hpDB[unit].sound)
				activeHP[unit] = t
			end
		end
	end
end

do
	local GetTime, UnitPower, UnitPowerMax = _G.GetTime, _G.UnitPower, _G.UnitPowerMax
	local r, p, t

	function MelodicVitals:VitalRepeatSetupPWR()
		r = ShivvAudio.db.profile.melodicVitals.repeatTimer
	end

	function MelodicVitals:UNIT_POWER(e, unit)
		if (not activePWR[unit]) then return end -- abort if not tracking

		p = UnitPower(unit) / UnitPowerMax(unit)

		if (p < pwrDB[unit].percent) then -- alert, its below threshold
			t = GetTime()

			if ((activePWR[unit] + r) < t and not UnitIsDeadOrGhost(unit)) then -- has it been long enough and are they living
				ShivvAudio:SoundHandler(pwrDB[unit].sound)
				activePWR[unit] = t
			end
		end
	end
end
